package com.gree.mall.miniapp.controller.common;

import cn.hutool.core.lang.TypeReference;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gree.mall.miniapp.annotation.ApiNotAuth;
import com.gree.mall.miniapp.bean.UmsPayCallBean;
import com.gree.mall.miniapp.constant.Constant;
import com.gree.mall.miniapp.exception.RemoteServiceException;
import com.gree.mall.miniapp.logic.common.WechatLogic;
import com.gree.mall.miniapp.logic.user.PayLogic;
import com.gree.mall.miniapp.utils.WeChatUtils;
import com.gree.mall.miniapp.annotation.ApiNotAuth;
import com.gree.mall.miniapp.exception.RemoteServiceException;
import com.gree.mall.miniapp.logic.user.PayLogic;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jdom.JDOMException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

@Slf4j
@RestController
@RequestMapping(value = "/pay", produces = "application/json; charset=utf-8")
public class PayController {

    @Autowired
    PayLogic payLogic;
    @Autowired
    WechatLogic wechatLogic;
    @Autowired
    RedisLockRegistry redisLockRegistry;
    static ExecutorService threadPool = Executors.newFixedThreadPool(4);



    @ApiNotAuth
    @PostMapping("/payCall")
    @ApiOperation(value = "支付回调", notes = "支付回调 ", httpMethod = "POST")
    public String payCall(@RequestBody String body) throws RemoteServiceException, JDOMException, IOException {

        Map<String, String> params = wechatLogic.doXMLParse(body);
        String id = params.get("out_trade_no");//订单号
        String transactionId = params.get("transaction_id");//微信支付单号
        BigDecimal totalFee = new BigDecimal(params.get("total_fee")).divide(BigDecimal.valueOf(100));//
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + id);

        threadPool.submit(() -> {
            try {
                log.info("==========支付回调开始：{}", body);
                if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
                    log.error("【支付回调】，请勿重复请求，orderId:{}",id);
                } else {
                    payLogic.payCall(id, transactionId, totalFee, null);
                }
            }catch(Exception e) {
                log.error("处理支付回调失败",e);
            }finally {
                if(obtain != null) {
                    obtain.unlock();
                }
            }
        });

        SortedMap<Object, Object> returnData = new TreeMap<>();
        returnData.put("return_code", "SUCCESS");
        returnData.put("return_msg", "OK");
        return WeChatUtils.getRequestXml(returnData);
    }


    @ApiNotAuth
    @PostMapping("/refundCall")
    @ApiOperation(value = "退款回调", notes = "退款回调 ", httpMethod = "POST")
    public String refundCall(@RequestBody String body) throws Exception {

       // String s = payLogic.refundCall(body);

        SortedMap<Object, Object> returnData = new TreeMap<>();
        returnData.put("return_code", "SUCCESS");
        returnData.put("return_msg", "OK");
        return WeChatUtils.getRequestXml(returnData);
    }

    @ApiNotAuth
    @PostMapping("/ums/payCall")
    @ResponseBody
    @ApiOperation(value = "云闪付回调", notes = "云闪付回调 ", httpMethod = "POST")
    public String umsPayCall(@ModelAttribute UmsPayCallBean bean) throws Exception  {
        log.info("云闪付支付回调 {}", JSONUtil.toJsonStr(bean));
        String id = JSONUtil.parseObj(bean.getAttachedData()).get("d", String.class);//订单号
        if (StringUtils.isBlank(id)) {
            log.error("云闪付回调异常");
            return "ERROR";
        }
        String transactionId = bean.getTargetOrderId();// 渠道订单号
        BigDecimal totalFee = new BigDecimal(bean.getTotalAmount()).divide(BigDecimal.valueOf(100));//
        Lock obtain = redisLockRegistry.obtain(Constant.RedisPrefix.LOCK_ORDER + id);

        threadPool.submit(() -> {
            try {
                if (!obtain.tryLock(10, TimeUnit.SECONDS)) {
                    log.error("【云闪付支付回调】，请勿重复请求，orderId:{}",id);
                } else {
                    payLogic.payCall(id, transactionId, totalFee, bean);
                }
            }catch(Exception e) {
                log.error("处理支付回调失败",e);
            }finally {
                if(obtain != null) {
                    obtain.unlock();
                }
            }
        });
        return "SUCCESS";
    }
}
