package com.gree.mall.miniapp.logic.common;

import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.model.GeneratePresignedUrlRequest;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.utils.StringUtils;
import com.gree.mall.miniapp.bean.user.CurrentCompanyWechat;
import com.gree.mall.miniapp.constant.Constant;
import com.gree.mall.miniapp.exception.RemoteServiceException;
import com.gree.mall.miniapp.helper.ResponseHelper;
import com.gree.mall.miniapp.plus.entity.*;
import com.gree.mall.miniapp.plus.service.*;
import com.gree.mall.miniapp.utils.CommonUtils;
import com.gree.mall.miniapp.utils.StringUtil;
import com.gree.mall.miniapp.utils.oss.OSSUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.*;
import java.net.URL;
import java.util.Date;
import java.util.List;
import java.util.UUID;

@Service
@Slf4j
public class CommonLogic {

    @Autowired
    CommonFileService commonFileService;
    @Autowired
    WxSceneService wxSceneService;
    @Autowired
    WxSceneDetailService wxSceneDetailService;
    @Autowired
    RegionService regionService;
    @Autowired
    UserService userService;
    @Autowired
    AdminCompanyWechatService adminCompanyWechatService;
    @Autowired
    AdminCompanyWechatOtherService adminCompanyWechatOtherService;
    @Autowired
    GoodsService goodsService;
    @Value("${spring.profiles.active}")
    private String profiles;
    @Value("${wechat.appid}")
    private String wechatAppid;
    @Value("${wechat.sub.appid}")
    private String wechatSubAppid;
    @Value("${wechat.sub.secret}")
    private String wechatSubSecret;

    @Autowired
    AdminWebsitService adminWebsitService;


    //高德的webkey
    private final String mapWebKey = "69cebe7cded9307a588237b0618856c9";


    // 用户相关文件夹
    public static final String ALIOSS_BUCKETNAME= "zfire-train";
    public static final String OSS_TYPE_PIC = "train/pic/";
    public static final String AliOSSEndPoint = "http://oss-cn-guangzhou.aliyuncs.com";
    public static final String accessUrl = "https://zfire-train.oss-cn-guangzhou.aliyuncs.com/";
//    public static final String OSSAccessKeyID = "LTAI4G3XTLh8G8DiTpXHpWgt";
//    public static final String OSSAccessKeySecret = "LYe8uf7Lg9WxJ1xejk45qnPFqxXLzN";

    /**
     * 获取当前用户
     */
    public CurrentCompanyWechat getCurrentCompanyWechat(HttpServletRequest request){


        String appid = CommonUtils.getAPPID(request);
        String userId = CommonUtils.getUserId(request);

        if(StringUtils.isEmpty(appid)){
            throw new RemoteServiceException("非法请求，缺少必须的参数");
        }
//        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.lambdaQuery()
//                .eq(AdminCompanyWechat::getSubAppId, appid)
//
//                .one();
//        if(adminCompanyWechat == null){
//            throw new RemoteServiceException("非法请求，请联系管理员配置必要的参数:"+appid);
//        }

        User user = userService.getById(userId);

        CurrentCompanyWechat currentCompanyWechat = new CurrentCompanyWechat();
//        BeanUtils.copyProperties(adminCompanyWechat,currentCompanyWechat);
        currentCompanyWechat.setCurrentCompanyWechatId("0");
        currentCompanyWechat.setCurrentCompanyName("平台");
        currentCompanyWechat.setAppId(wechatAppid);
        currentCompanyWechat.setMchId(null);
        currentCompanyWechat.setMchKey(null);
        currentCompanyWechat.setSubAppId(wechatSubAppid);
        currentCompanyWechat.setSubSecret(wechatSubSecret);
        currentCompanyWechat.setUserId(CommonUtils.getUserId(request));
        currentCompanyWechat.setUser(user);


        if (user != null && !StringUtil.isEmpty(user.getWebsitId())) {
            AdminWebsit adminWebsit = adminWebsitService.getById(user.getWebsitId());
            currentCompanyWechat.setAdminWebsit(adminWebsit);
        }
        return currentCompanyWechat;

    }

    /**
     * 获取当前企业
     */
    public AdminCompanyWechat getAdminCompanyWechat(String companyWechatId){
        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById(companyWechatId);
        return adminCompanyWechat;
    }

    /**
     * 微信scene
     */
    public String getSceneValue(String wxSceneId){
        WxScene wxScene = this.getScene(wxSceneId);
        return wxScene == null ? "" : wxScene.getParam();
    }

    /**
     * 微信scene
     */
    public WxScene getScene(String wxSceneId){
        WxScene wxScene = wxSceneService.getById(wxSceneId);
        if(!wxScene.getStatus()){
            throw new RemoteServiceException("链接无效");
        }
        if(wxScene.getEndTime().getTime() < new Date().getTime()){
            throw new RemoteServiceException("链接已过期");
        }
        return wxScene;
    }

    /**
     * 微信scene详情
     */
    public List<WxSceneDetail> getSceneDetail(String wxSceneId){
        List<WxSceneDetail> list = wxSceneDetailService.lambdaQuery().eq(WxSceneDetail::getWxSceneId, wxSceneId).list();
        return list;
    }

    /**
     * 附件归属绑定
     */
    public void bindFile(String objId,String objType,List<String> fileIds){
        if(fileIds.size() == 0){
            return;
        }
        //先删除后绑定
        commonFileService.lambdaUpdate()
                .eq(CommonFile::getObjId,objId)
                .eq(CommonFile::getObjType,objType)
                .notIn(CommonFile::getId,fileIds).remove();

        commonFileService.lambdaUpdate()
                .set(CommonFile::getObjId,objId)
                .set(CommonFile::getObjType,objType)
                .in(CommonFile::getId,fileIds).update();
    }

    /**
     * 查询附件列表
     */
    public List<CommonFile> queryFileByObjId(String objId,String objType){
        return commonFileService.lambdaQuery()
                .eq(CommonFile::getObjId,objId)
                .eq(CommonFile::getObjType,objType)
                .orderByAsc(CommonFile::getCreateTime)
                .list();
    }

    /**
     * 查询街道列表
     */
    public List<Map> street(String province,String city,String area){
        List<Map> street = getStreet(province, area);
        if(street.size() == 0){
            Map map = new HashMap();
            map.put("name",area);
            street.add(map);
        }
        return street;
    }


    public Map<String, String> getOSSConfig() throws UnsupportedEncodingException {
        Map<String, String> config = OSSUtil.getConfig();
        return config;
    }

    public String getFile(String key, HttpServletResponse response) throws IOException {
        String url = OSSUtil.getUrlWw(key);
        return url;
    }

    /**
     * 获取高德行政地址
     */
    public List<Map> getStreet(String province,String area){
        try {
            String s = HttpUtil.get("https://restapi.amap.com/v3/config/district?keywords=" + province + "&subdistrict=1&key=" + mapWebKey);
            JSONObject jsonObject = JSONObject.parseObject(s);
            String status = (String)jsonObject.get("status");
            if(status.equals("1")){
                List<Object> districts = (List<Object>) jsonObject.get("districts");
                String adcode = (String)((Map<String,Object>)districts.get(0)).get("adcode");

                String street = HttpUtil.get("https://restapi.amap.com/v3/config/district?keywords=" + area + "&subdistrict=1&key=" + mapWebKey+"&adcode="+adcode);
                Object districts1 = JSONObject.parseObject(street).get("districts");
                List<Map> maps = JSONArray.parseArray(JSONObject.toJSONString(districts1), Map.class);

                List<Map> districts2 = JSONArray.parseArray(JSONObject.toJSONString(maps.get(0).get("districts")), Map.class);
                return districts2;
            }
        }catch(Exception e){
            return new ArrayList<>();
        }
        return new ArrayList<>();
    }

    /**
     * 上传附件
     * @param file
     * @return
     * @throws IOException
     */
    public CommonFile uploadFile(MultipartFile file) throws IOException, RemoteServiceException {

        if (file == null || file.isEmpty()) {
            return null;
        }

        String uploadFilePath = file.getOriginalFilename().replaceAll("\\\\", "/");
        String uploadFileName = uploadFilePath.substring(
                uploadFilePath.lastIndexOf('/') + 1, uploadFilePath.lastIndexOf('.'));
        String uploadFileSuffix = uploadFilePath.substring(uploadFilePath.lastIndexOf('.') + 1, uploadFilePath.length());
        long currentTime = System.currentTimeMillis();
        BufferedOutputStream bufferedOutputStream = null;

        //限制大小为10m
        if (file.getSize() > 1024 * 1024 * 30) {
            throw new RemoteServiceException("文件大小超出限制");
        }

        try {
            String filePath = OSSUtil.OSS_TYPE_PIC + currentTime + UUID.randomUUID().toString() + "." + uploadFileSuffix;
            if (!OSSUtil.uploadFile(filePath, file.getBytes())) {
                throw new RemoteServiceException("文件同步oss失败");
            }
            String ossUrl = OSSUtil.accessUrl + filePath;

            CommonFile commonFile = new CommonFile();
            commonFile.setUrl(ossUrl);
            commonFile.setName(uploadFileName);
            commonFile.setFileType(uploadFileSuffix);
            commonFile.setFileSize(BigDecimal.valueOf(file.getSize()));
            commonFile.setCreateTime(new Date());
            commonFileService.save(commonFile);

            return commonFile;
        } finally {
            if (bufferedOutputStream != null) {
                bufferedOutputStream.close();
            }
        }

    }

    public String getFileGet(String key) {
        String url = OSSUtil.getUrlWw(key,null);
        return url;
    }






}
