package com.gree.mall.miniapp.logic.common;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gree.mall.miniapp.bean.PayDetail;
import com.gree.mall.miniapp.bean.UmsMiniRepBean;
import com.gree.mall.miniapp.enums.UMSDiscountCodeEnum;
import com.gree.mall.miniapp.enums.UMSGZDiscountCodeEnum;
import com.gree.mall.miniapp.enums.UMSNotGZDiscountCodeEnum;
import com.gree.mall.miniapp.exception.RemoteServiceException;
import com.gree.mall.miniapp.logic.AppletOrders;
import com.gree.mall.miniapp.plus.entity.AdminCompanyWechat;
import com.gree.mall.miniapp.plus.entity.AdminWebsit;
import com.gree.mall.miniapp.plus.entity.OrderDetail;
import com.gree.mall.miniapp.plus.entity.OrderInfo;
import com.gree.mall.miniapp.plus.service.AdminCompanyWechatService;
import com.gree.mall.miniapp.plus.service.AdminWebsitService;
import com.gree.mall.miniapp.utils.ArithUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.apache.http.HttpEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.math.BigDecimal;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Objects;
import java.util.UUID;

@Service
@Slf4j
public class UMSLogic {

    @Resource
    AdminCompanyWechatService adminCompanyWechatService;
    @Resource
    AdminWebsitService adminWebsitService;
    @Value("${ums.url}")
    private String url;
    @Value("${ums.payment.notifyUrl}")
    private String notifyUrl;
    @Value("${wechat.sub.appid}")
    private String wechatSubAppid;

    /**
     * 云闪付 todo
     * @param orderInfo
     * @param payment
     * @param companyWechatId
     * @param orderDetail
     * @param ip
     * @return
     * @throws Exception
     */
    public PayDetail payment(OrderInfo orderInfo, BigDecimal payment, String openid, String companyWechatId, OrderDetail orderDetail, String ip) throws Exception {
        if(payment.doubleValue() <= 0){
            PayDetail payDetail = new PayDetail();
            payDetail.setIsPay(false);
            payDetail.setId(orderInfo.getOrderId());
            return payDetail;
        }

        final AdminCompanyWechat company = adminCompanyWechatService.getById(companyWechatId);
        final AdminWebsit websit = adminWebsitService.getById(orderInfo.getWebsitId());
        JSONObject json = new JSONObject();

        StringBuilder allAddress = new StringBuilder();
        if (!orderInfo.getReceAddress().contains(orderInfo.getProvince())) {
            allAddress.append(orderInfo.getProvince());
        }
        if (!orderInfo.getReceAddress().contains(orderInfo.getCity())) {
            allAddress.append(orderInfo.getCity());
        }
        if (!orderInfo.getReceAddress().contains(orderInfo.getArea())) {
            allAddress.append(orderInfo.getArea());
        }
        allAddress.append(orderInfo.getReceAddress());

        json.put("requestTimestamp", DateFormatUtils.format(new Date(), "yyyy-MM-dd HH:mm:ss"));    // 报文请求时间
        json.put("merOrderId", websit.getYunSystem() + orderInfo.getOrderId()); // 银商分配的4位来源编号 + 商户订单号
        json.put("mid", websit.getYunCompany()); // 商户号
        json.put("tid", websit.getYunNumber());    // 终端号
        json.put("instMid", "MINIDEFAULT");
        json.put("totalAmount", (int) ArithUtils.mul(payment.doubleValue(), 100));      // 支付总金额
        json.put("subAppId", wechatSubAppid);    // 微信子商户appId
        json.put("subOpenId", openid);    // 微信子商户openid
        json.put("tradeType", "UP_WX_MINI");    // 交易类型
        json.put("invokeScene", "03");    // 交易发起场景
        json.put("notifyUrl", notifyUrl);  // 支付结果通知地址
        json.put("name", Base64.encodeBase64String(orderInfo.getBuyerName().getBytes(StandardCharsets.UTF_8)));    // 实名认证姓名
        json.put("mobile", Base64.encodeBase64String(orderInfo.getBuyerMobile().getBytes(StandardCharsets.UTF_8)));    // 实名认证手机号
        json.put("certNo", Base64.encodeBase64String(orderInfo.getBuyerIdCard().getBytes(StandardCharsets.UTF_8)));  // 实名认证证件号
        json.put("certType", "IDENTITY_CARD");  // 实名认证证件类型
        json.put("fixBuyer", "T");  // 是否需要实名认证 需要实名认证时置为T
        json.put("srcReserve", allAddress.toString());  // 请求系统预留字段
        json.put("clientIp", ip);  // 客户端IP 用户客户端的ip地址
        if (StringUtils.isNotBlank(websit.getYunTax())) {
            json.put("ylyxId", websit.getYunTax().substring(3));    // 二级商编
        }
        if (StringUtils.isNotBlank(company.getCompanyName())) {
            json.put("ylyxName", company.getCompanyName());    // 二级商名称
        }
        if (StringUtils.isNotBlank(websit.getYunName())) {
            json.put("ylyxMerAbbr", websit.getYunName());    // 二级商简称
        }
        JSONObject attachedDataJson = new JSONObject();
        attachedDataJson.put("ISO2O", websit.getYunTwo().equals("YES") ? "Y" : "N");
        attachedDataJson.put("SPcode", websit.getYunTwo().equals("YES") ? websit.getYunSpCode() : "");
        attachedDataJson.put("d", orderInfo.getOrderId());
        json.put("attachedData", attachedDataJson);


        if (Objects.nonNull(orderDetail) && StringUtils.isNotBlank(orderDetail.getUmsDiscountCode()) && StringUtils.isNotBlank(orderDetail.getBarCode())) {
            json.put("discountCode", orderDetail.getUmsDiscountCode());
            if (!websit.getYunTwo().equals("YES")) {
                final String coverByCode = UMSNotGZDiscountCodeEnum.findCoverByCode(orderDetail.getUmsDiscountCode());
                if (StringUtils.isNotBlank(coverByCode)) {
                    json.put("discountCode", coverByCode);
                }
            } else if (websit.getYunTwo().equals("YES")) {
                final String coverByCode = UMSGZDiscountCodeEnum.findCoverByCode(orderDetail.getUmsDiscountCode());
                if (StringUtils.isNotBlank(coverByCode)) {
                    json.put("discountCode", coverByCode);
                }
            }

            JSONArray array = new JSONArray();
            JSONObject goods = new JSONObject();
            String goodsName = orderDetail.getGoodsName().replace("以旧换新", "")
                    .replace("一级能效", "")
                    .replace("二级能效", "")
                    .replace("格力", "")
                    .replace("品牌", "")
                    .replace("【", "")
                    .replace("】", "")
                    .replaceAll(" ", "");
            goods.put("goodsCategory", UMSDiscountCodeEnum.findShortNameByCode(orderDetail.getUmsDiscountCode()));
            goods.put("goodsId", orderDetail.getBarCode());
            goods.put("quantity", orderDetail.getNum());
            goods.put("goodsName", goodsName.length() > 120 ? goodsName.substring(0, 120) : goodsName);
            goods.put("body", UMSDiscountCodeEnum.findNameByCode(orderDetail.getUmsDiscountCode()).contains("一级") ? "一级" : "二级");
            goods.put("price", (int) ArithUtils.mul(orderDetail.getPrice().doubleValue(), 100));
            array.add(goods);
            json.put("goods", array);
        }

        //OPEN-BODY-SIG 方式
        String authorization = getOpenBodySig(websit.getYunAppid(), websit.getYunAppkey(), json.toString());
//        String authorization = AppletOrders.getOpenBodySig(company.getUmsAppid(), company.getUmsAppKey(), json.toString());
        String send = send(url + "/v1/netpay/uac/mini-order", json.toString() , authorization);
        final UmsMiniRepBean resultObj = JSONObject.parseObject(send, UmsMiniRepBean.class);
        final String errCode = resultObj.getErrCode();
        if (!errCode.equals("SUCCESS")) {
            throw new RemoteServiceException("请求云闪付失败, 平台错误信息: " + resultObj.getErrMsg());
        }

        PayDetail payDetail = new PayDetail();
        payDetail.setId(orderInfo.getOrderId());
        payDetail.setUmsMiniRep(resultObj);
        log.info("payDetail:{}",JSONObject.toJSONString(payDetail));
        return payDetail;
    }

    /**
     * open-body-sig方式获取到Authorization 的值
     *
     * @param appId  f0ec96ad2c3848b5b810e7aadf369e2f
     * @param appKey 775481e2556e4564985f5439a5e6a277
     * @param body   json字符串 String body = "{\"merchantCode\":\"123456789900081\",\"terminalCode\":\"00810001\",\"merchantOrderId\":\"20123333644493200\",\"transactionAmount\":\"1\",\"merchantRemark\":\"测试\",\"payMode\":\"CODE_SCAN\",\"payCode\":\"285668667587422761\",\"transactionCurrencyCode\":\"156\"}";
     * @return
     * @throws Exception
     */
    private String getOpenBodySig(String appId, String appKey, String body) throws Exception {
        String timestamp = new SimpleDateFormat("yyyyMMddHHmmss").format(new Date());   // eg:20190227113148
        String nonce = UUID.randomUUID().toString().replace("-", ""); // eg:be46cd581c9f46ecbd71b9858311ea12
        byte[] data = body.getBytes(StandardCharsets.UTF_8);
        log.info("ums data:\n" + body);
        InputStream is = new ByteArrayInputStream(data);
        String bodyDigest = DigestUtils.sha256Hex(is); // eg:d60bc3aedeb853e2a11c0c096baaf19954dd9b752e48dea8e919e5fb29a42a8d
        log.info("ums bodyDigest:\n" + bodyDigest);
        String str1_C = appId + timestamp + nonce + bodyDigest; // eg:f0ec96ad2c3848b5b810e7aadf369e2f + 20190227113148 + be46cd581c9f46ecbd71b9858311ea12 + d60bc3aedeb853e2a11c0c096baaf19954dd9b752e48dea8e919e5fb29a42a8d

        log.info("ums str1_C:" + str1_C);
        byte[] localSignature = hmacSHA256(str1_C.getBytes(), appKey.getBytes());

        String localSignatureStr = Base64.encodeBase64String(localSignature);   // Signature
        log.info("ums Authorization:\n" + "OPEN-BODY-SIG AppId=" + "\"" + appId + "\"" + ", Timestamp=" + "\"" + timestamp + "\"" + ", Nonce=" + "\"" + nonce + "\"" + ", Signature=" + "\"" + localSignatureStr + "\"\n");
        return ("OPEN-BODY-SIG AppId=" + "\"" + appId + "\"" + ", Timestamp=" + "\"" + timestamp + "\"" + ", Nonce=" + "\"" + nonce + "\"" + ", Signature=" + "\"" + localSignatureStr + "\"");
    }

    /**
     * @param data
     * @param key
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeyException
     */
    private byte[] hmacSHA256(byte[] data, byte[] key) throws NoSuchAlgorithmException, InvalidKeyException {
        String algorithm = "HmacSHA256";
        Mac mac = Mac.getInstance(algorithm);
        mac.init(new SecretKeySpec(key, algorithm));
        return mac.doFinal(data);
    }

    /**
     * 发送请求
     * @param url
     * @return
     * @throws Exception
     */
    private String send(String url, String entity , String authorization) throws Exception {
        CloseableHttpClient httpClient = HttpClients.createDefault();
        HttpPost httpPost = new HttpPost(url);
        httpPost.addHeader("Authorization", authorization);
        StringEntity se = new StringEntity(entity, "UTF-8");
        se.setContentType("application/json");
        httpPost.setEntity(se);
        CloseableHttpResponse response = httpClient.execute(httpPost);
        HttpEntity entity1 = response.getEntity();
        String resStr = null;
        if (entity1 != null) {
            resStr = EntityUtils.toString(entity1, "UTF-8");
        }
        log.info("云闪付接口返回字符串: {}", resStr);
        httpClient.close();
        response.close();
        return resStr;
    }
}



