package com.gree.mall.miniapp.logic.goods;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.miniapp.bean.goods.CategoryBean;
import com.gree.mall.miniapp.bean.goods.GoodsNewBean;
import com.gree.mall.miniapp.bean.user.CurrentCompanyWechat;
import com.gree.mall.miniapp.commonmapper.CategoryMapper;
import com.gree.mall.miniapp.commonmapper.SecActivityStageMapper;
import com.gree.mall.miniapp.logic.common.CommonLogic;
import com.gree.mall.miniapp.plus.entity.*;
import com.gree.mall.miniapp.plus.service.GoodsCategoryService;
import com.gree.mall.miniapp.plus.service.GoodsService;
import com.gree.mall.miniapp.plus.service.GoodsTagRelaService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CategoryLogic {

    @Autowired
    private GoodsService goodsService;
    @Autowired
    private CategoryMapper categoryMapper;
    @Autowired
    GoodsCategoryService goodsCategoryService;
    @Autowired
    SecActivityStageMapper secActivityStageMapper;
    @Autowired
    CommonLogic commonLogic;

    @Autowired
    GoodsTagRelaService goodsTagRelaService;

    /**
     * 分类列表
     * @return
     */
    public List<GoodsCategory> list(HttpServletRequest request,String parentId,String name){
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);


        List<GoodsCategory> list = goodsCategoryService.lambdaQuery()
                .eq(StringUtils.isNotEmpty(parentId), GoodsCategory::getParentId, parentId)
                .eq(StringUtils.isEmpty(parentId), GoodsCategory::getLevel, 1)
                .like(StringUtils.isNotEmpty(name),GoodsCategory::getName,name)
                .eq(GoodsCategory::getStatus, true)
                .eq(GoodsCategory::getDel,false)
                .orderByDesc(GoodsCategory::getSortNum)
                .orderByDesc(GoodsCategory::getCreateTime)
                .list();
        return list;
    }


    public List<CategoryBean> queryAll(HttpServletRequest request){
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        return  categoryMapper.queryCategory(currentCompanyWechat.getCurrentCompanyWechatId());
    }
    public List<GoodsCategory> queryFirst(HttpServletRequest request){
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        return goodsCategoryService.lambdaQuery()
                .isNull(GoodsCategory::getParentId)
                .eq(GoodsCategory::getStatus,true)
                .eq(GoodsCategory::getDel,false)
                .orderByAsc(GoodsCategory::getSortNum)
                .orderByDesc(GoodsCategory::getCreateTime)
                .list();
    }
    public List<GoodsCategory> querySecond(HttpServletRequest request , String categoryId){
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        return goodsCategoryService.lambdaQuery()
                .eq(GoodsCategory::getParentId,categoryId)
                .eq(GoodsCategory::getStatus,true)
                .eq(GoodsCategory::getDel,false)
                .orderByAsc(GoodsCategory::getSortNum)
                .orderByDesc(GoodsCategory::getCreateTime)
                .list();
    }


    public IPage<GoodsNewBean> categoryGoodsList(HttpServletRequest request,String categoryId, Integer sort,Integer pageNum, Integer pageSize) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        if (StringUtils.isEmpty(categoryId))
            categoryId=null;

        IPage<GoodsNewBean> page = categoryMapper.listGoods(new Page(pageNum, pageSize),currentCompanyWechat.getCurrentCompanyWechatId(),categoryId,sort);

        List<GoodsNewBean> records = new ArrayList<>();

        //补充标签
        for (Goods bean : page.getRecords()) {
            GoodsNewBean goodsNewBean = BeanUtil.toBean(bean, GoodsNewBean.class);
            //查询上标签
            List<GoodsTagRela> list1 = goodsTagRelaService.lambdaQuery().eq(GoodsTagRela::getGoodsId, bean.getGoodsId())
                    .eq(GoodsTagRela::getType,1).list();
            if (list1.size() > 0) {
                List<String> collect1 = list1.stream().map(GoodsTagRela::getGoodsTagName).collect(Collectors.toList());
                goodsNewBean.setTags1(collect1);
            }

            //查询活动专区标签 //查询下标签
            //活动商品
            List<GoodsTagRela> list = goodsTagRelaService.lambdaQuery().eq(GoodsTagRela::getGoodsId, bean.getGoodsId())
                    .eq(GoodsTagRela::getType,2).list();
            if (list.size() > 0) {
                List<String> collect1 = list.stream().map(GoodsTagRela::getGoodsTagName).collect(Collectors.toList());
                goodsNewBean.setTags2(collect1);
            }
            records.add(goodsNewBean);
        }
        page.setRecords(records);

        return page;
    }


    public IPage<Goods> categoryGoodsList(HttpServletRequest request,String categoryId, Integer pageNum, Integer pageSize) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        List<GoodsCategory> goodsCategories = goodsCategoryService.lambdaQuery()
                .eq(GoodsCategory::getParentId, categoryId)
                .list();
        List<String> categoryIds = goodsCategories.stream().map(GoodsCategory::getCategoryId).collect(Collectors.toList());
        if(categoryIds.size() == 0){
            return new Page<>(pageNum,pageSize);
        }

        return goodsService.lambdaQuery()
                .in(Goods::getCategoryId,categoryIds)
                .eq(Goods::getStatus,true)
                .eq(Goods::getDel,false)
                .eq(Goods::getPromotionGroup,false)
                .orderByDesc(Goods::getSortNum)
                .orderByDesc(Goods::getSoldNum)
                .orderByDesc(Goods::getCreateTime)
                .page(new Page(pageNum, pageSize));
    }


}
