package com.gree.mall.miniapp.logic.goods;

import com.gree.mall.miniapp.bean.goods.GoodsCategoryBean;
import com.gree.mall.miniapp.commonmapper.CustomGoodsCategoryMapper;
import com.gree.mall.miniapp.plus.entity.GoodsCategory;
import com.gree.mall.miniapp.plus.service.GoodsCategoryService;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Component
public class GoodsCategoryLogic {

    @Resource
    CustomGoodsCategoryMapper customGoodsCategoryMapper;
    @Resource
    GoodsCategoryService goodsCategoryService;

    /**
     * 分类管理列表
     * @param level
     * @param status
     * @return
     */
    public List<GoodsCategoryBean> list(Integer level, Boolean status) {
        return customGoodsCategoryMapper.list(level, status);
    }

    /**
     * 新增分类
     * @param goodsCategoryBean
     */
    @Transactional(rollbackFor = Exception.class)
    public void add(GoodsCategoryBean goodsCategoryBean) {
        Date createDate = new Date();
        if (goodsCategoryBean.getChildren() != null) {
            List<GoodsCategory> goodsCategoryList = goodsCategoryBean.getChildren()
                    .stream()
                    .map(c -> c.setCreateTime(createDate))
                    .collect(Collectors.toList());
            goodsCategoryService.saveBatch(goodsCategoryList);
        } else {
            goodsCategoryBean.setCreateTime(createDate);
            goodsCategoryService.save(goodsCategoryBean);
        }
    }

    /**
     * 修改分类
     * @param goodsCategory
     */
    @Transactional(rollbackFor = Exception.class)
    public void update(GoodsCategory goodsCategory) {
        goodsCategoryService.updateById(goodsCategory);
    }

    /**
     * 删除分类
     * @param categoryId
     */
    @Transactional(rollbackFor = Exception.class)
    public void delete(String categoryId) {
        goodsCategoryService.lambdaUpdate()
                .set(GoodsCategory::getDel, true)
                .eq(GoodsCategory::getCategoryId, categoryId)
                .or()
                .eq(GoodsCategory::getParentId, categoryId)
                .update();
    }


    public GoodsCategory queryById(String catagoryId){
        return goodsCategoryService.getById(catagoryId);
    }


}
