package com.gree.mall.miniapp.logic.goods;

import cn.binarywang.wx.miniapp.bean.live.WxMaLiveResult;
import cn.hutool.core.bean.BeanUtil;
import com.alibaba.excel.util.CollectionUtils;
import com.aliyuncs.utils.StringUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.miniapp.bean.goods.*;
import com.gree.mall.miniapp.bean.user.CurrentCompanyWechat;
import com.gree.mall.miniapp.commonmapper.GoodsSpecDetailMapper;
import com.gree.mall.miniapp.commonmapper.SecActivityStageMapper;
import com.gree.mall.miniapp.constant.Constant;
import com.gree.mall.miniapp.enums.GoodsTypeEnum;
import com.gree.mall.miniapp.enums.QrCodeEnum;
import com.gree.mall.miniapp.enums.UserTypeEnum;
import com.gree.mall.miniapp.exception.RemoteServiceException;
import com.gree.mall.miniapp.logic.common.CommonLogic;
import com.gree.mall.miniapp.logic.common.WechatLogic;
import com.gree.mall.miniapp.logic.seckill.SecKillLogic;
import com.gree.mall.miniapp.plus.entity.*;
import com.gree.mall.miniapp.plus.service.*;
import com.gree.mall.miniapp.utils.StringUtil;
import com.gree.mall.miniapp.utils.oss.OSSUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;


@Component
public class GoodsLogic {
    @Resource
    CommonLogic commonLogic;
    @Autowired
    WechatLogic wechatLogic;
    @Autowired
    SecKillLogic secKillLogic;
    @Resource
    GoodsService goodsService;
    @Resource
    GoodsSpecService goodsSpecService;
    @Autowired
    GoodsPackagePopService goodsPackagePopService;

    @Autowired
    private GoodsFavoriteService goodsFavoriteService;
    @Autowired
    private GoodsSpecDetailMapper goodsSpecDetailMapper;
    @Autowired
    private PubTemplateService pubTemplateService;
    @Autowired
    PromotionGroupSpecUserService promotionGroupSpecUserService;
    @Autowired
    PromotionGroupSpecService promotionGroupSpecService;
    @Autowired
    UserService userService;
    @Autowired
    PromotionGroupService promotionGroupService;
    @Autowired
    SecActivityStageMapper secActivityStageMapper;

    @Autowired
    GoodsTagRelaService goodsTagRelaService;

    @Autowired
    GoodsApplyService goodsApplyService;

    @Autowired
    GoodsApplyItemService goodsApplyItemService;

    /**
     * 商品列表
     *
     * @param keyword
     * @param pageNum
     * @param pageSize
     * @return
     */
    public IPage<GoodsNewBean> page(HttpServletRequest request,String keyword,String categoryId, Integer sort,Integer pageNum, Integer pageSize) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        IPage<GoodsNewBean> page = goodsSpecDetailMapper.queryGoodsList(new Page(pageNum, pageSize),
                currentCompanyWechat.getUser().getUserId(), currentCompanyWechat.getUser().getType(),
                keyword, categoryId,sort,null);

        this.supplyGoodsNewBean(page,currentCompanyWechat);
        return page;
    }



    public void supplyGoodsNewBean(IPage<GoodsNewBean> page, CurrentCompanyWechat currentCompanyWechat){
        //判定是否存在价格调整 在就覆盖其价格
        for (GoodsNewBean goodsNewBean : page.getRecords()) {
            if(goodsNewBean == null){
                continue;
            }
            GoodsApply goodsApply = goodsApplyService.lambdaQuery().eq(GoodsApply::getGoodsId, goodsNewBean.getGoodsId())
                    .eq(GoodsApply::getWebsitId, currentCompanyWechat.getUser().getWebsitId())
                    .eq(GoodsApply::getStatus, "OK")
                    .last("limit 1").one();
            if(goodsApply != null){
                goodsNewBean.setOrgGoodsPrice(goodsApply.getOrgGoodsPrice());
                goodsNewBean.setGoodsPrice(goodsApply.getGoodsPrice());
            }
        }
    }

    /**
     * 商品详情
     */
    public GoodsBean detail(HttpServletRequest request, String goodsId,String userId,String wxSceneId){
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);

        List<PromotionGroupSpecUser> list = new ArrayList<>();

        User user = userService.getById(userId);
        if(user.getPromotionGroupLeader()){
            //团长在该活动商品的规格
           list = promotionGroupSpecUserService.lambdaQuery()
                    .eq(PromotionGroupSpecUser::getUserId,userId)
                    .eq(PromotionGroupSpecUser::getGoodsId,goodsId)
                    .list();
        }else{
            String promotionGroupUserId = user.getPromotionGroupUserId();
            if(StringUtils.isNotEmpty(promotionGroupUserId)) {
                list = promotionGroupSpecUserService.lambdaQuery()
                        .eq(PromotionGroupSpecUser::getUserId, promotionGroupUserId)
                        .eq(PromotionGroupSpecUser::getGoodsId, goodsId)
                        .list();
            }
        }

        GoodsBean goodsBean = null;
        if(list.size() == 0){
            goodsBean = this.commonDetail(goodsId, userId,wxSceneId);
        }else{
            goodsBean = this.groupGoodsdetail(list);
        }
        //记录用户浏览记录
        GoodsVisit goodsVisit = new GoodsVisit();
        goodsVisit.setGoodsId(goodsId);
        goodsVisit.setGoodsName(goodsBean.getGoodsName());
        goodsVisit.setUserId(userId);
        goodsVisit.setUserName(user.getNickName());
        goodsVisit.setUserPhone(user.getMobile());
        goodsVisit.setPromotionGroupId(goodsBean.getPromotionGroupId());
        goodsVisit.setGoodsPrice(goodsBean.getGoodsPrice());
        goodsVisit.setShareUserId(goodsBean.getGroupUserId());
        goodsVisit.setCompanyId(currentCompanyWechat.getCurrentCompanyWechatId());
        goodsVisit.setCompanyName(currentCompanyWechat.getCurrentCompanyName());
        goodsVisit.setCreateTime(new Date());
        goodsVisit.insert();

        return goodsBean;
    }


    /**
     * 商品详情
     *
     * @param goodsId
     * @return
     */
    public GoodsBean commonDetail(String goodsId, String userId,String wxSceneId) throws RemoteServiceException {
        User user = userService.getById(userId);
        String serviceId = user.getServiceId();
        if(user.getType().equals(UserTypeEnum.SERVICE.toString())){
            serviceId = user.getUserId();
        }
        User service = userService.getById(serviceId);
        Goods goods = goodsService.lambdaQuery()
                .eq(Goods::getGoodsId, goodsId)
                .eq(Goods::getDel, false)
                .eq(Goods::getStatus, true)
                .eq(Goods::getPromotionGroup,false)
                .one();
        if (goods == null) {
            throw new RemoteServiceException("商品不存在或已下架");
        }
        //商品规格（包含秒杀商品）
        List<GoodsSpecSecBean> goodsSpecs = goodsSpecDetailMapper.querySpecSec(goodsId);

        //替换商家价格
        GoodsApply goodsApply = goodsApplyService.lambdaQuery()
                .eq(GoodsApply::getGoodsId, goods.getGoodsId())
                .eq(GoodsApply::getWebsitId, user.getWebsitId())
                .eq(GoodsApply::getStatus, "OK")
                .last("limit 1").one();
        if (goodsApply != null){
            List<GoodsApplyItem> goodsApplyItems =
                    goodsApplyItemService.lambdaQuery()
                    .eq(GoodsApplyItem::getGoodsApplyId, goodsApply.getGoodsApplyId())
                    .list();

            goods.setGoodsPrice(goodsApply.getGoodsPrice());
            goods.setOrgGoodsPrice(goodsApply.getOrgGoodsPrice());

            for (GoodsSpecSecBean goodsSpec : goodsSpecs) {
                List<GoodsApplyItem> goodsApplyItemList = goodsApplyItems.stream().filter(item -> item.getGoodsSpecId().equals(goodsSpec.getGoodsSpecId()))
                        .collect(Collectors.toList());

                if (!CollectionUtils.isEmpty(goodsApplyItemList)){
                    goodsSpec.setPrice(goodsApplyItemList.get(0).getApplyPrice());
                    goodsSpec.setShareAmount(goodsApplyItemList.get(0).getApplyShareAmount());
                    goodsSpec.setInnerShareAmount(goodsApplyItemList.get(0).getApplyInnerShareAmount());
                }
            }
        }

        //是否有收藏
        Integer goodsFavoritesCount = goodsFavoriteService.lambdaQuery()
                .eq(GoodsFavorite::getGoodsId, goodsId)
                .eq(GoodsFavorite::getUserId, userId)
                .count();

        //商品库存
        Integer goodsStock = goodsSpecs.stream().collect(Collectors.summingInt(x -> x.getStockNum()));
        GoodsBean goodsBean = new GoodsBean();
        BeanUtils.copyProperties(goods, goodsBean);

        //内部人员、秒杀活动、让利分享 等对价格分佣金额产生的影响处理
        goodsSpecs.forEach(o -> {
            if(o.getOrgPrice().doubleValue() == 0) {
                o.setOrgPrice(o.getOrgGoodsPrice());
            }
            //内部人员
            if (service != null && service.getInnerr() && service.getType().equals(UserTypeEnum.SERVICE.toString())) {
                o.setShareAmount(o.getInnerShareAmount());
            }
            //秒杀
            if (o.getSecType()) {
                o.setPrice(o.getSecPrice());
                o.setShareAmount(o.getSecShareAmount());
            }
        });
        //让利
        Map<String, BigDecimal> cutMap = this.cutAmountGoods(wxSceneId);
        if(cutMap != null){
            for (GoodsSpec o : goodsSpecs) {
                //让利
                BigDecimal cutAmount = cutMap.get(o.getGoodsSpecId());
                if(cutAmount != null){
                    o.setShareAmount(o.getShareAmount().subtract(cutMap.get(o.getGoodsSpecId())));
                    o.setPrice(o.getPrice().subtract(cutAmount));
                }
            }
        }
        if(!CollectionUtils.isEmpty(goodsSpecs)) {
            //秒杀商品，需要覆盖一些值
            if (goodsSpecs.get(0).getSecType()) {
                goodsBean.setSecType(goodsSpecs.get(0).getSecType());
                goodsBean.setLimitBuy(goodsSpecs.get(0).getLimitBuy());
                goodsBean.setSecStockNum(goodsSpecs.get(0).getSecStockNum());
                goodsBean.setSecKillId(goodsSpecs.get(0).getSecKillId());
                goodsBean.setSecKillSpecId(goodsSpecs.get(0).getSecKillSpecId());
                goodsBean.setEndHour(goodsSpecs.get(0).getEndHour());
                goodsBean.setGoodsPrice(goodsSpecs.get(0).getSecPrice());
            }
            goodsBean.setShareAmount(goodsSpecs.get(0).getShareAmount());
        }
        if(goodsBean.getGoodsType().equals(GoodsTypeEnum.PACKAGE.toString())){
            goodsBean.setShareAmount(goodsBean.getPackageMinShareAmount());
            goodsBean.setGoodsPrice(goodsBean.getPackageMinAmount());
            goodsBean.setOrgGoodsPrice(goodsBean.getPackageMinAmount());
        }
        goodsBean.setGoodsSpecs(goodsSpecs);
        goodsBean.setImages(commonLogic.queryFileByObjId(goods.getGoodsId(), Constant.Img.GOODS_IMG));
        goodsBean.setFavorite(goodsFavoritesCount > 0);
        goodsBean.setStock(goodsStock);
        goodsBean.setCommonTemplate(goodsSpecDetailMapper.queryCommonTemplate(goodsId));
        goodsBean.setPubCommonTemplate(pubTemplateService.getById(goods.getFreightTemplateId()));

        return goodsBean;
    }


    /**
     * 返回 降价的金额幅度map key=商品规格id，value = 降价的金额
     * @param wxSceneId
     * @return
     */
    public Map<String,BigDecimal> cutAmountGoods(String wxSceneId){
        //通过分享的形式购买的
        Map<String,BigDecimal> cutMap = new HashMap<>();
        if(StringUtils.isNotEmpty(wxSceneId)) {
            WxScene scene = commonLogic.getScene(wxSceneId);
            if(scene.getType().equals(QrCodeEnum.CUT.toString().toLowerCase())) {
                //如果是秒杀商品，不在秒杀活动中则无效
                if(StringUtils.isNotEmpty(scene.getSecKillActivityId()) && !secKillLogic.isSecing(scene.getSecKillActivityId())){
                    throw new RemoteServiceException("分享链接活动已结束");
                }
                //如果不是秒杀商品，但在秒杀活动中则无效
                if(StringUtils.isEmpty(scene.getSecKillActivityId()) && secKillLogic.isSecing(scene.getSecKillActivityId())){
                    throw new RemoteServiceException("分享链接已过期");
                }
                List<WxSceneDetail> sceneDetail = commonLogic.getSceneDetail(wxSceneId);
                for(WxSceneDetail wxSceneDetail : sceneDetail){
                    cutMap.put(wxSceneDetail.getGoodsSpecId(),wxSceneDetail.getCutAmount());
                }
            }
        }

        if(cutMap.keySet().size() == 0){
            return null;
        }
        return cutMap;
    }

    /**
     * 团购商品列表
     * @param userId
     * @return
     */
    public IPage<PromotionGoodsBean> myGroupGoods(String userId,String goodsCategoryId, String keyword, Integer pageNum, Integer pageSize,HttpServletRequest request){
        User user = userService.getById(userId);
        if(!user.getPromotionGroupLeader()){
            userId = user.getPromotionGroupUserId();
        }
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String companyWechatId = currentCompanyWechat.getCompanyWechatId();
        IPage<PromotionGoodsBean> promotionGoodsBeanIPage = goodsSpecDetailMapper.queryPromotionGoods(new Page(pageNum, pageSize),
                userId,goodsCategoryId, keyword,null);
        return promotionGoodsBeanIPage;
    }

    /**
     * 团购商品详情
     */
    public GoodsBean groupGoodsdetail(List<PromotionGroupSpecUser> list){

        PromotionGroupSpecUser promotionGroupSpecUser = list.get(0);

        //团长id
        String groupUserId = promotionGroupSpecUser.getUserId();
        User groupUser = userService.getById(groupUserId);
        if(!groupUser.getPromotionGroupLeader()){
            throw new RemoteServiceException("链接已失效");
        }

        Goods goods = goodsService.getById(promotionGroupSpecUser.getGoodsId());

        GoodsBean goodsBean = new GoodsBean();
        BeanUtils.copyProperties(goods,goodsBean);
        //多规格
        List<GoodsSpecSecBean> goodsSpecSecBeans = new ArrayList<>();
        for(int i=0;i<list.size();i++){
            PromotionGroupSpecUser specUser = list.get(i);
            PromotionGroupSpec promotionGroupSpec = promotionGroupSpecService.getById(specUser.getPromotionGroupSpecId());

            GoodsSpecSecBean specSecBean = new GoodsSpecSecBean();
            specSecBean.setPrice(specUser.getGroupPrice());
            specSecBean.setOrgPrice(specUser.getOrgGoodsPrice());
            specSecBean.setImgUrl(specUser.getGoodsImgSrc());
            specSecBean.setShareAmount(specUser.getShareAmount());
            specSecBean.setStockNum(promotionGroupSpec.getStock());
            specSecBean.setSoldNum(promotionGroupSpec.getSalesNum());
            specSecBean.setName(promotionGroupSpec.getGoodsSpecName());
            specSecBean.setSpecValue(promotionGroupSpec.getGoodsSpecValue());
            specSecBean.setGoodsSpecId(promotionGroupSpec.getGoodsSpecId());
            specSecBean.setGoodsId(promotionGroupSpec.getGoodsId());

            if(i == 0) {
                goodsBean.setStock(promotionGroupSpec.getStock());
                goodsBean.setSoldNum(promotionGroupSpec.getSalesNum());
                goodsBean.setPromotionGroupId(promotionGroupSpec.getPromotionGroupId());
            }
            goodsSpecSecBeans.add(specSecBean);
        }
        goodsBean.setOrgGoodsPrice(promotionGroupSpecUser.getOrgGoodsPrice());
        goodsBean.setGoodsPrice(promotionGroupSpecUser.getGroupPrice());
        goodsBean.setImages(commonLogic.queryFileByObjId(goods.getGoodsId(), Constant.Img.GOODS_IMG));
        goodsBean.setGroupUserId(groupUserId);
        goodsBean.setGroupUserName(promotionGroupSpecUser.getUserName());
        goodsBean.setGroupPic(groupUser.getAvatar());
        goodsBean.setGoodsSpecs(goodsSpecSecBeans);
        return goodsBean;
    }
    /**
     * 团购活动二维码
     */
    public PromotionShareQrCode queryShareQrcode(String userId,HttpServletRequest request) throws IOException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        User user = userService.getById(currentCompanyWechat.getUserId());
        if(!user.getStatus() || !user.getPromotionGroupLeader()){
            throw new RemoteServiceException("无效的用户");
        }
        //查询活动背景图
        List<PromotionGroup> list = promotionGroupService.lambdaQuery()
                .le(PromotionGroup::getStartTime, new Date())
                .ge(PromotionGroup::getEndTime, new Date())
                .eq(PromotionGroup::getStatus, true)
                .eq(PromotionGroup::getCompanyId,user.getCompanyId())
                .orderByDesc(PromotionGroup::getCreateTime)
                .list();
        if(list.size() == 0){
            return null;
        }
        PromotionGroup promotionGroup = list.get(0);
        String qrcode = wechatLogic.getQrcode(QrCodeEnum.PROMOTION_GROUP.toString().toLowerCase(),promotionGroup.getPromotionGroupId(),userId,user.getCompanyId());
        PromotionShareQrCode promotionShareQrCode = new PromotionShareQrCode();
        promotionShareQrCode.setPromotionImgUrl(qrcode);
        promotionShareQrCode.setQrcode(qrcode);
        promotionShareQrCode.setPromotionImgUrl(promotionGroup.getImgUrl());
        promotionShareQrCode.setPosterImgUrl(promotionGroup.getPosterImgUrl());
        return promotionShareQrCode;
    }


    /**
     * 让利二维码或链接
     */
    public String getCutQrcode(List<CutGoodsBean> cutGoodsBeans, HttpServletRequest request){
        if(CollectionUtils.isEmpty(cutGoodsBeans)){
            throw new RemoteServiceException("降价二维码参数异常");
        }
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String userId = currentCompanyWechat.getUserId();
        String companyWechatId = currentCompanyWechat.getCompanyWechatId();
        String secKillActivityId = cutGoodsBeans.get(0).getSecKillActivityId();
        String goodsId = cutGoodsBeans.get(0).getGoodsId();
        Boolean qrcode = cutGoodsBeans.get(0).getQrcode();
        User user = userService.getById(userId);
        //检查让利是否合理
        Goods goods = this.getGoodsById(goodsId);
        if(goods.getPromotionGroup()){
            throw new RemoteServiceException("团购商品不支持降价分享");
        }
        //检查合法性
        for(CutGoodsBean cutGoodsBean : cutGoodsBeans){
            this.checkCut(user, cutGoodsBean.getGoodsId(), cutGoodsBean.getGoodsSpecId(), cutGoodsBean.getCutAmount(), companyWechatId);
        }
        String cutQrcode = wechatLogic.getCutQrcode(goodsId,userId, companyWechatId, qrcode,secKillActivityId, cutGoodsBeans);
        return cutQrcode;
    }

    /**
     * 判断让利金额是否合
     */
    public Boolean checkCut(User user,String goodsId,String goodsSpecId,BigDecimal cutAmount,String companyWechatId){
        if(!user.getType().equals(UserTypeEnum.SERVICE.toString())){
            throw new RemoteServiceException("非分销员不可降价分享");
        }
        //覆盖其上秒杀价格
        SecKillActivitySpec secKillActivitySpec = secActivityStageMapper.getSecKillActivitySpecByGoodId(goodsId);
        if(secKillActivitySpec != null){
            //秒杀的分佣金额
            BigDecimal amount = secKillActivitySpec.getAmount();
            if(amount.doubleValue() < cutAmount.doubleValue()){
                throw new RemoteServiceException("让利降价金额设置不合理");
            }
        }else {
            //检查普通商品
            GoodsSpec goodsSpec = this.getGoodsSpecById(goodsSpecId);
            //内部人员价格
            boolean innerrUserShare = user.getInnerr() && goodsSpec.getInnerShareAmount().doubleValue() < cutAmount.doubleValue();
            //外部人员价格
            boolean notInnerrUserShare = !user.getInnerr() && goodsSpec.getShareAmount().doubleValue() < cutAmount.doubleValue();
            if(innerrUserShare || notInnerrUserShare){
                throw new RemoteServiceException("让利降价金额设置不合理");
            }
        }

        return true;
    }

    /**
     * 获取商品
     * @param goodsId
     * @return
     */
    public Goods getGoodsById(String goodsId) {
        return goodsService.getById(goodsId);
    }

    public GoodsSpec getGoodsSpecById(String goodsSpecId) {
        return goodsSpecService.getById(goodsSpecId);
    }

    /**
     * 团购商品-选购列表
     */
    public List<List<GoodsPackageBean>> choicePackage(String goodsId,HttpServletRequest request){
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String companyWechatId = currentCompanyWechat.getCompanyWechatId();
        List<GoodsPackageBean> goodsPackageBeans = goodsSpecDetailMapper.queryGoodsPackage(currentCompanyWechat.getUser().getUserId(),
                currentCompanyWechat.getUser().getType(),goodsId, null);
        if(CollectionUtils.isEmpty(goodsPackageBeans)){
            return new ArrayList();
        }
        List<Integer> types = goodsPackageBeans.stream().map(GoodsPackageBean::getType).distinct().collect(Collectors.toList());
        List<List<GoodsPackageBean>> list = new ArrayList<>();
        for(Integer type : types){
            List<GoodsPackageBean> collect = goodsPackageBeans.stream().filter(v -> v.getType() == type).collect(Collectors.toList());
            list.add(collect);
        }
        return list;
    }

    public String updateMergeImage(String goodsId, String imgUrl) {
        if (StringUtils.isNotEmpty(goodsId)) {
            goodsService.lambdaUpdate()
                    .set(Goods::getMergeImage, OSSUtil.accessUrl + imgUrl)
                    .eq(Goods::getGoodsId, goodsId)
                    .update();
            return OSSUtil.accessUrl + imgUrl;
        }
        return "";
    }
}
