package com.gree.mall.miniapp.logic.notice;


import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.gree.mall.miniapp.bean.notice.NoticeParamBean;
import com.gree.mall.miniapp.commonmapper.WxCustomerMapper;
import com.gree.mall.miniapp.plus.entity.MpMaRela;
import com.gree.mall.miniapp.plus.entity.Notice;
import com.gree.mall.miniapp.plus.entity.NoticeParam;
import com.gree.mall.miniapp.plus.entity.OrderRequest;
import com.gree.mall.miniapp.plus.service.MpMaRelaService;
import com.gree.mall.miniapp.plus.service.NoticeParamService;
import com.gree.mall.miniapp.plus.service.NoticeService;
import com.gree.mall.miniapp.plus.service.OrderRequestService;
import com.gree.mall.miniapp.utils.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.api.WxMpTemplateMsgService;
import me.chanjar.weixin.mp.api.WxMpUserService;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import me.chanjar.weixin.mp.bean.result.WxMpUserList;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateData;
import me.chanjar.weixin.mp.bean.template.WxMpTemplateMessage;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.*;

@Slf4j
@Component
public class NoticeLogic {


    //订阅模板id
//    public static final String ORDER_NOTICE_TEMPLATE = "TzbPSrqTmQEegEHVWkJCBVBirtPLAG6pxGKL2pNB0G4";

    @Value("${spring.profiles.active}")
    private String env;
//    @Value("${wx.mp.app-id}")
//    private String wxMpAppid;
//    @Value("${wx.mp.secret}")
//    private String wxMpSecret;
    @Value("${wechat.sub.appid}")
    private String subAppId;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    MpMaRelaService mpMaRelaService;
    @Autowired
    WxCustomerMapper wxCustomerMapper;
    @Autowired
    OrderRequestService orderRequestService;
    @Autowired
    private NoticeService noticeService;

    @Resource
    private WxMpService wxMpService;
    @Autowired
    NoticeParamService noticeParamService;

    //每次重发的间隔时间
    final Integer[] errMin = {1,2,5,10,20,40,80,160,320};
    final Integer errNum = errMin.length;
    /**
     * 新版通知v2-保存参数
     */
    public void addNotice(com.gree.mall.miniapp.bean.notice.NoticeParam noticeParam){
        NoticeParam noticeParam1 = BeanUtil.copyProperties(noticeParam, NoticeParam.class);
        noticeParam1.setCreateTime(new Date());
        noticeParam1.setStatus(false);
        noticeParam1.setErrNum(0);
        noticeParam1.insert();
    }

    //新工单通知
    public void v2Push3(){
        final String templateId = "JKutPIfWeO-wHUA2yl5QSrK7a4Tvkt-hRma4m3TwScQ";
        final Integer type = 4;
        //只查询昨晚20点到今天的
        DateTime dateTime = DateUtil.offsetDay(DateUtil.beginOfDay(new Date()), -1);
        DateTime startTime = DateUtil.offsetHour(dateTime, 20);

        List<NoticeParamBean> list = wxCustomerMapper.queryNoticeParam(type, errNum, DateUtil.formatDateTime(startTime));
        if(list.isEmpty()){
            return;
        }
        for(NoticeParamBean noticeParam : list) {
            Map<String, String> map = new HashMap<>();
            map.put("character_string2", noticeParam.getParam1());
            map.put("const1",noticeParam.getParam2());
            map.put("time6",noticeParam.getParam3());
            map.put("thing7",noticeParam.getParam4());

            this.pushV2(noticeParam,templateId,map,"");
        }
    }


    /**
     * 新版通知v2-发送通知
     */
    public void pushV2(NoticeParamBean bean,String templateId,Map<String,String> map,String pagePath){
        //final String templateId = "LkYvqT8JC1zQWcUs8znIaT5Rj1WNGiBJ_eikedZ3XtI";
        log.info("【工单通知】通知类型:{}",bean.getType());

        //判断有失败的情况间隔的时间达到要求没
        if(bean.getErrNum() >= errNum)
            return;
        if(bean.getErrNum() > 0) {
            Integer min = errMin[bean.getErrNum() - 1];
            DateTime newLastTime = DateUtil.offsetMinute(bean.getLastTime(), min);
            if(newLastTime.getTime() > new Date().getTime())
                return;
        }
        List<WxMpTemplateData> data = supplyV2(map);
        //MpMaRela one = mpMaRelaService.lambdaQuery().eq(MpMaRela::getMaPhone, bean.getMobile()).last("limit 1").one();

        Boolean push = this.push(bean.getMpOpenId(), templateId, data, pagePath);
        if(push){
            bean.setStatus(true);
        }else{
            bean.setErrNum(bean.getErrNum() + 1);
        }

        bean.setLastTime(new Date());
        bean.updateById();

    }



    public List<WxMpTemplateData> supplyV2(Map<String,String> map){
        List<WxMpTemplateData> data = Lists.newArrayList();
        for (String s : map.keySet()) {
            WxMpTemplateData keyword0 = new WxMpTemplateData(s,map.get(s));
            data.add(keyword0);
        }
        return data;
    }







    /**
     * 家盛茂系统内部消息通知
     * @param title
     * @param content
     * @param objId
     * @param type
     * @return
     */
    public Notice add(String title, String content, String objId, Integer type) {
        Notice notice = new Notice();
        notice.setTitle(title);
        notice.setContent(content);
        notice.setObjId(objId);
        notice.setType(type);
        notice.setReadFlag(false);
        notice.setCreateTime(new Date());
        noticeService.save(notice);
        return notice;
    }




    /**
     * 推送公众号模板通知
     * @param openId 公众号的模板id
     * @param data 模板内容
     * @param pagepath 跳转小程序路径
     */
    private Boolean push(String openId, String templateId, List<WxMpTemplateData> data, String pagepath){
        if(!env.equals("prd")){
            return true;
        }
        WxMpTemplateMsgService templateMsgService = wxMpService.getTemplateMsgService();
        WxMpTemplateMessage templateMessage = new WxMpTemplateMessage();
        templateMessage.setToUser(openId);
        templateMessage.setTemplateId(templateId);
        templateMessage.setData(data);

        if(StringUtils.isEmpty(pagepath)){
            pagepath = "pages/mine/workOrder/list";
        }
//        if(env.equals("prd")) {
//            WxMpTemplateMessage.MiniProgram miniProgram = new WxMpTemplateMessage.MiniProgram(subAppId, pagepath, true);
//            templateMessage.setMiniProgram(miniProgram);
//        }

        try {
            templateMsgService.sendTemplateMsg(templateMessage);
        } catch (WxErrorException e) {
            log.error("模板消息推送失败", e);
            return false;
        }
        return true;
    }



    /**
     *  同步openid
     */
    public void syncOpenId() {
//        log.info("同步家盛茂公众号0");
        if(!env.equals("prd")){
            return;
        }
//        log.info("同步家盛茂公众号1");
        //查询最近的1个openid
        MpMaRela lastMpMaRela = mpMaRelaService.lambdaQuery()
                .orderByDesc(MpMaRela::getCreateTime)
                .last("limit 1")
                .one();

        WxMpUserService wxMpUserService = wxMpService.getUserService();
//        log.info("同步家盛茂公众号2");
        //获取已关注的用户列表
        String nextOpenId = Optional.ofNullable(lastMpMaRela)
                .map(MpMaRela::getMpOpenId)
                .orElse(null);

        WxMpUserList wxMpUserList;
        try {
            nextOpenId = null;
//            log.info("同步家盛茂公众号3");
            wxMpUserList = wxMpUserService.userList(nextOpenId);
        } catch (WxErrorException e) {
            log.error("同步家盛茂公众号信息失败.", e);
            return;
        }
        List<String> openIdList = Optional.ofNullable(wxMpUserList)
                .map(WxMpUserList::getOpenids)
                .orElse(Collections.emptyList());

        //查询详情

        //检查下是库里是否有内容，如果有则同步最近5天的，否则同步所有
        int count = mpMaRelaService.count();
        boolean all = count == 0;
        //5天前
        long sub5DayTime = DateUtil.offsetDay(new Date(), -5).getTime() / 1000;
//        log.info("同步家盛茂公众号4:");
        //开发者可通过该接口来批量获取用户基本信息。最多支持一次拉取100条。
        List<List<String>> split = ListUtil.split(openIdList, 100);
        for (List<String> v : split) {
            List<MpMaRela> mpMaRelas = new ArrayList<>();
            List<WxMpUser> wxMpUsers;
            try {
                wxMpUsers = wxMpUserService.userInfoList(v);
            } catch (WxErrorException e) {
                log.error("同步家盛茂公众号信息失败", e);
                return;
            }
//            log.info("【同步家盛茂公众号信息】:{}", wxMpUsers);
            for (WxMpUser obj :wxMpUsers) {
                String openId = obj.getOpenId();
                String unionId = obj.getUnionId();
                Boolean subscribe = obj.getSubscribe();
                Long subscribeTime = obj.getSubscribeTime();
                if(StringUtils.isEmpty(openId) || StringUtils.isEmpty(unionId) || !subscribe){
                    //不需要处理为关注的用户
                    continue;
                }
//                if(subscribeTime < sub5DayTime && !all){
//                    continue;
//                }
                MpMaRela mpMaRela = new MpMaRela();
                mpMaRela.setCreateTime(new Date());
                mpMaRela.setMpOpenId(openId);
                mpMaRela.setUnionId(unionId);
                mpMaRelas.add(mpMaRela);
            }
//            log.info("同步家盛茂公众号5:"+mpMaRelas.size());
            //处理只需要保存近5天的内容
            if(!mpMaRelas.isEmpty()) {
                mpMaRelaService.saveOrUpdateBatch(mpMaRelas);
            }
        }

        wxCustomerMapper.updateMpMaRela();
    }
}
