package com.gree.mall.miniapp.logic.user;

import cn.binarywang.wx.miniapp.api.WxMaService;
import cn.binarywang.wx.miniapp.bean.WxMaPhoneNumberInfo;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.IdUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.miniapp.bean.user.*;
import com.gree.mall.miniapp.commonmapper.*;
import com.gree.mall.miniapp.constant.Constant;
import com.gree.mall.miniapp.enums.OrderShareStatusEnum;
import com.gree.mall.miniapp.enums.QrCodeEnum;
import com.gree.mall.miniapp.enums.UserTypeEnum;
import com.gree.mall.miniapp.exception.RemoteServiceException;
import com.gree.mall.miniapp.helper.ResponseHelper;
import com.gree.mall.miniapp.logic.LbsAmapLogic;
import com.gree.mall.miniapp.logic.common.CommonLogic;
import com.gree.mall.miniapp.logic.common.WechatLogic;
import com.gree.mall.miniapp.logic.coupon.CouponLogic;
import com.gree.mall.miniapp.logic.order.ShoppingCartLogic;
import com.gree.mall.miniapp.plus.entity.*;
import com.gree.mall.miniapp.plus.service.*;
import com.gree.mall.miniapp.utils.*;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.cp.bean.Gender;
import me.chanjar.weixin.cp.bean.WxCpMaJsCode2SessionResult;
import me.chanjar.weixin.cp.bean.WxCpUser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.integration.redis.util.RedisLockRegistry;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;
import java.util.stream.Collectors;

/**
 * 小程序用户
 */
@Slf4j
@Service
public class UserLogic {

    @Autowired
    CommonLogic commonLogic;
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    WechatLogic wechatLogic;
    @Autowired
    ShoppingCartLogic shoppingCartLogic;
    @Autowired
    CouponLogic couponLogic;
    @Autowired
    RedisLockRegistry redisLockRegistry;
    @Autowired
    UserService userService;
    @Autowired
    OrderShareService orderShareService;
    @Autowired
    UserVisitService userVisitService;
    @Autowired
    EnterpriseWechatWorkerService enterpriseWechatWorkerService;
    @Autowired
    private OrderShareRankMapper orderShareRankMapper;
    @Autowired
    private UserInfoMapper userInfoMapper;
    @Autowired
    private MyProfitMapper myProfitMapper;
    @Autowired
    private CustomWebsitNumberMapper customWebsitNumberMapper;
    @Autowired
    PromotionGroupSpecUserService promotionGroupSpecUserService;
    @Autowired
    UserServiceWaitService userServiceWaitService;
    @Autowired
    AdminWebsitService adminWebsitService;

    @Value("${wechat.sub.appid}")
    private String wechatSubAppid;

    @Autowired
    LbsAmapLogic lbsAmapLogic;

    /**
     * 微信授权
     *
     * @param code
     * @param serviceId
     * @return
     * @throws RemoteServiceException
     * @throws InterruptedException
     */
    @Transactional
    public UserWxBean addUser(String code, String serviceId,HttpServletRequest request) throws RemoteServiceException, InterruptedException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        Map<String, Object> returnMap = wechatLogic.authToken(code,currentCompanyWechat);
        String openId = (String) returnMap.get("openid");
        String sessionKey = (String) returnMap.get("session_key");
        String unionId = (String) returnMap.get("unionid");

        log.info("【微信授权】：{}", JSONObject.toJSONString(returnMap));

        String key = Constant.RedisPrefix.LOCK_AUTH + ":" + openId;
        Lock obtain = redisLockRegistry.obtain(key);
        if (!obtain.tryLock(2, TimeUnit.SECONDS)) {
            log.error("请勿重复授权");
            return null;
        }
        try {
            //如果用户不存在则新注册
            User user = userService.lambdaQuery()
                    .eq(User::getOpenId, openId)
                    .one();
            if (user == null) {
                user = new User();
                user.setLastLoginTime(new Date());
                user.setNickName("微信用户_" + IdUtil.objectId().substring(16, 24));
                user.setOpenId(openId);
                user.setSessionKey(sessionKey);
                user.setServiceId(serviceId);
                user.setServiceTime(new Date());
                user.setLastLoginTime(new Date());
                user.setCreateTime(new Date());
                user.setCompanyId(currentCompanyWechat.getCurrentCompanyWechatId());
                user.setCompanyName(currentCompanyWechat.getCompanyName());
                userService.save(user);
                log.info("【注册成为会员】：{}", user);
                return this.supplyUser(user);
            }

            //如果用户存在，则判断是否为业务员，并更新相关信息
            if (!user.getStatus()) {
                throw new RemoteServiceException("用户已被冻结");
            }
            //判断该手机号是否为业务员
            user = userToService(user, currentCompanyWechat);

            user.setUnionId(unionId);
            user.setSessionKey(sessionKey);
            user.setLastLoginTime(new Date());
            if (StringUtils.isEmpty(user.getServiceId()) && StringUtils.isNotEmpty(serviceId)) {
                user.setServiceId(serviceId);
                user.setServiceTime(new Date());
            }
            user.setOpenId(openId);
            user.updateById();
            return this.supplyUser(user);
        } finally {
            obtain.unlock();
        }

    }

    public User userToService(User user,CurrentCompanyWechat currentCompanyWechat){
        //判断该手机号是否为业务员
        if (StringUtils.isNotEmpty(user.getMobile())) {
//            List<EnterpriseWechatWorker> list = enterpriseWechatWorkerService.lambdaQuery()
//                    .eq(EnterpriseWechatWorker::getMobile, user.getMobile())
//                    .eq(EnterpriseWechatWorker::getCorpId, currentCompanyWechat.getCorpId())
//                    .list();
            UserServiceWait waitUser = userServiceWaitService.lambdaQuery()
                    .eq(UserServiceWait::getMobile, user.getMobile())
                    .eq(UserServiceWait::getCompanyId, user.getCompanyId())
                    .one();

            if (Objects.nonNull(waitUser)) {
//                String workUserId = list.get(0).getWechatUserId();
                Integer count = userService.lambdaQuery()
                        .eq(User::getMobile, user.getMobile())
                        .eq(User::getType, UserTypeEnum.SERVICE.toString())
                        .eq(User::getCompanyId, user.getCompanyId())
                        .count();
                if (count < 1) {
                    user.setType(UserTypeEnum.SERVICE.toString());
                    user.setWorkName(waitUser.getName());
                    user.setWorkUserId(waitUser.getWorkerId());
                    user.setPosition(waitUser.getPosition());
                    user.setWorkPhone(waitUser.getMobile());
//                    user.setWorkQrcode(list.get(0).getQrCode());
                    user.setWebsitId(waitUser.getWebsitId());
                    user.setWebsitName(waitUser.getWebsitName());
                    user.setCompanyId(currentCompanyWechat.getCurrentCompanyWechatId());
                    user.setCompanyName(currentCompanyWechat.getCompanyName());

                    waitUser.setUserId(user.getUserId())
                            .setIsBind(true)
                            .setUpdateTime(new Date())
                            .updateById();

                }
            }
        }
        return user;
    }

    /**
     * 返回小程序用户登录需要的信息
     *
     * @param user
     * @return
     */
    private UserWxBean supplyUser(User user) {
        UserWxBean userWxBean = new UserWxBean();
        String token = JwtUtils.createJWT(user.getUserId(), user.getOpenId(), 7 * 24 * 60 * 60 * 1000);
        BeanUtils.copyProperties(user, userWxBean);
        userWxBean.setToken(token);
        userWxBean.setShoppingCartNums(shoppingCartLogic.total(userWxBean.getUserId()));

        return userWxBean;
    }

    /**
     * 绑定业务员
     */
    public void bind(String userId, String serviceId) {
        User user = userService.getById(userId);
        User service = userService.getById(serviceId);
        //绑定业务员
        if (user.getType().equals(UserTypeEnum.GENERAL.toString()) && service.getType().equals(UserTypeEnum.SERVICE.toString())) {
            user.setServiceId(serviceId);
            user.setServiceTime(new Date());
            user.setWebsitId(service.getWebsitId());
            user.setWebsitName(service.getWebsitName());
            user.setCompanyId(service.getCompanyId());
            user.setCompanyName(service.getCompanyName());
        }

        //判断是否为团长
        if(!user.getPromotionGroupLeader()) {
            if (service.getPromotionGroupLeader()) {
                user.setPromotionGroupUserId(serviceId);
            }else{
                String promotionGroupUserId = service.getPromotionGroupUserId();
                user.setPromotionGroupUserId(promotionGroupUserId);
            }
        }

        user.updateById();

    }

    public User getUserByMobile(String mobile,String companyWechatId){
        return userService.lambdaQuery()
                .eq(User::getMobile,mobile)
                .eq(User::getStatus,true)
                .eq(User::getCompanyId,companyWechatId).one();
    }

    /**
     * 获取绑定业务员小程序码
     */
    public String bindQrCode(HttpServletRequest request) throws RemoteServiceException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String userId = CommonUtils.getUserId(request);
        User user = userService.lambdaQuery()
                .eq(User::getUserId, userId)
                .eq(User::getStatus,true)
                .one();
        if (user == null) {
            throw new RemoteServiceException("用户不存在");
        }
        if (StringUtils.isNotEmpty(user.getBindQrcode())) {
            return user.getBindQrcode();
        }
        String qrcode = null;
        try {
            qrcode = wechatLogic.getQrcode(QrCodeEnum.BIND.toString().toLowerCase(Locale.ROOT),
                    user.getWorkUserId(), user.getUserId(),currentCompanyWechat.getCompanyWechatId());
            if (StringUtils.isNotEmpty(qrcode)) {
                user.setBindQrcode(qrcode);
                userService.updateById(user);
            }
        } catch (Exception e) {
            log.error("获取绑定业务员小程序码失败", e);
        }
        return qrcode;

    }


    public RankBean rank(HttpServletRequest request) throws RemoteServiceException {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        String userId = CommonUtils.getUserId(request);
        User user = userService.lambdaQuery()
                .eq(User::getUserId, userId)
                .eq(User::getStatus,true)
                .one();
        if (user == null || StringUtils.isEmpty(user.getWorkUserId())) {
            throw new RemoteServiceException("业务员不存在");
        }

        Date now = new Date();
        Date endTime = DateConputeUtils.getCurrentMonthStart(now);
        Date startTime = DateConputeUtils.getLastMonthStart(now);
        Date endTimeDay = DateConputeUtils.getCurrentDayStart(now);
        RankBean rankBean = new RankBean();
        List<RankItemBean> rankItemBeans = orderShareRankMapper.queryRankList(null,endTime, endTimeDay,currentCompanyWechat.getCompanyWechatId());
        List<RankItemBean> myRanks = null;
        if (rankItemBeans != null && rankItemBeans.size() > 0) {
            rankBean.setRankList(rankItemBeans.size() > 10 ? rankItemBeans.subList(0, 10) : rankItemBeans);
            //此处的workUserId就是userId
            myRanks = rankItemBeans.stream()
                    .filter(o -> o.getWorkUserId() != null && o.getWorkUserId().equals(user.getUserId()))
                    .limit(1)
                    .collect(Collectors.toList());
        }
        RankItemBean myrank = myRanks != null && myRanks.size() > 0 ? myRanks.get(0) : orderShareRankMapper.queryWorkerInfo(user.getUserId(),currentCompanyWechat.getCompanyWechatId());
        rankBean.setMyRank(myrank);

        BigDecimal previousIncome = new BigDecimal("0.00");
        BigDecimal currentIncome = new BigDecimal("0.00");
        List<RankItemBean> previous = orderShareRankMapper.queryRankList(user.getUserId(), startTime, endTime,currentCompanyWechat.getCompanyWechatId());
        List<RankItemBean> current = orderShareRankMapper.queryRankList(user.getUserId(), endTime, endTimeDay,currentCompanyWechat.getCompanyWechatId());
        if (previous != null && previous.size() > 0) {
            previousIncome = previous.get(0).getTotal();
        }
        if (current != null && current.size() > 0) {
            currentIncome = current.get(0).getTotal();
        }

        rankBean.setUpOrDown(currentIncome.compareTo(previousIncome) > 0 ? true : false);
        rankBean.setIncome(currentIncome);
        Calendar date = Calendar.getInstance();
        date.setTime(endTimeDay);
        //date.add(Calendar.SECOND, -1);

        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm");
        rankBean.setStartPeriod(simpleDateFormat.format(endTime));
        rankBean.setEndPeriod(simpleDateFormat.format(date.getTime()));
        return rankBean;
    }

    /**
     * 解除绑定
     */
    public void unBind() {
        //满30天自动解绑
        Date timeByDay = DateUtils.getTimeByDay(new Date(), -30);
        userService.lambdaUpdate().set(User::getServiceId, "").set(User::getServiceTime, null)
                .ne(User::getServiceId, "")
                .lt(User::getServiceTime, timeByDay)
                .eq(User::getType, UserTypeEnum.GENERAL.toString())
                .update();
    }


    //手机号码授权
    @Transactional
    public UserWxBean getWxMobile(String iv, String encryptedData, String userId,HttpServletRequest request) throws Exception {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        UserWxBean userWxBean = new UserWxBean();
        User user = this.userService.getById(userId);
        if (null == user) {
            throw new RemoteServiceException("用户信息为空" + userId);
        }
        WxMaService maService = wechatLogic.getMaService(wechatSubAppid);
        WxMaPhoneNumberInfo phoneNoInfo = maService.getUserService().getPhoneNoInfo(user.getSessionKey(), encryptedData, iv);
        String mobile = phoneNoInfo.getPhoneNumber();
        if(StringUtils.isEmpty(mobile)){
            throw new RemoteServiceException("手机号授权失败");
        }
//        byte[] resultByte = AesCbcUtil.decrypt(Base64.decodeBase64(encryptedData), Base64.decodeBase64(user.getSessionKey()), Base64.decodeBase64(iv));
//        if (null == resultByte || resultByte.length < 0) {
//            throw new RemoteServiceException("手机号授权失败");
//        }
//        String userInfo = new String(resultByte, "UTF-8");
//        Map<String, Object> map = Utils.stringToMap(userInfo);
//        log.info("授权手机号response:{}", JSONObject.toJSONString(map));
//        //获取手机号码
//        String mobile = String.valueOf(map.get("phoneNumber"));

        //判断该手机号是否为业务员
//        List<EnterpriseWechatWorker> list = enterpriseWechatWorkerService.lambdaQuery()
//                .eq(EnterpriseWechatWorker::getMobile, mobile)
//                .eq(EnterpriseWechatWorker::getCorpId,currentCompanyWechat.getCorpId())
//                .list();
        List<UserServiceWait> waitList = userServiceWaitService.lambdaQuery()
                .eq(UserServiceWait::getCompanyId, currentCompanyWechat.getCompanyWechatId())
                .eq(UserServiceWait::getMobile, mobile)
                .list();

        //根据手机号查询是否存在企业微信用户
        User workUser = this.getUserByMobile(mobile,currentCompanyWechat.getCompanyWechatId());
        if (workUser == null) {
            //企业微信用户为空，则直接授权手机号。并根据手机号判断是否存在于微信成员表如果存在则为业务员
            user.setMobile(mobile);
            if (waitList.size() > 0) {
                UserServiceWait wait = waitList.get(0);
                String workUserId = wait.getWorkerId();
                Integer count = userService.lambdaQuery()
                        .eq(User::getWorkUserId, workUserId)
                        .eq(User::getType, UserTypeEnum.SERVICE.toString())
                        .eq(User::getCompanyId,currentCompanyWechat.getCompanyWechatId())
                        .count();
                if (count < 1) {
                    user.setType(UserTypeEnum.SERVICE.toString());
                    user.setWorkName(wait.getName());
                    user.setWorkUserId(workUserId);
                    user.setWorkPhone(wait.getMobile());
//                    user.setWorkQrcode(list.get(0).getQrCode());
                    user.setPosition(wait.getPosition());
                    user.setWorkCreateTime(new Date());

                    wait.setUserId(user.getUserId())
                            .setIsBind(true)
                            .setUpdateTime(new Date())
                            .updateById();
                }
            }
            user.updateById();

            String token = JwtUtils.createJWT(user.getUserId(), user.getOpenId(), 7 * 24 * 60 * 60 * 1000);
            BeanUtils.copyProperties(user, userWxBean);
            userWxBean.setToken(token);

        } else {
            //企业微信不为空，则将数据绑定到企业微信,并删除授权openid的用户数据
            workUser.setOpenId(user.getOpenId());
            workUser.updateById();
            user.deleteById();

            String token = JwtUtils.createJWT(workUser.getUserId(), workUser.getOpenId(), 7 * 24 * 60 * 60 * 1000);
            BeanUtils.copyProperties(workUser, userWxBean);
            userWxBean.setToken(token);
        }
        return userWxBean;
    }


    //保存用户信息
    public User saveUserInfo(String userId, String nickName, String sex, String avatarUrl, String country, String province, String city) {
        User user = new User();
        user.setUserId(userId);
        user.setNickName(nickName);
        user.setSex(sex);
        user.setAvatar(avatarUrl);
        user.setCountry(country);
        user.setProvince(province);
        user.setCity(city);
        user.updateById();
        return user;
    }


    /**
     * 个人信息
     */
    public UserWxBean userDetail(String userId,HttpServletRequest request,String lat,String lng) throws RemoteServiceException {
        User user = userService.getById(userId);
        if (user == null) {
            throw new RemoteServiceException(ResponseHelper.ResponseCode_AUTH_ERROR, "请求不合法");
        }

        UserWxBean userWxBean = new UserWxBean();
        BeanUtils.copyProperties(user, userWxBean);

        userWxBean.setShoppingCartNums(shoppingCartLogic.total(userWxBean.getUserId()));
        // TODO 逻辑需要调整
//        userWxBean.setServiceWorkerBean(userInfoMapper.queryWebsit(user.getUserId()));
        if (!StringUtil.isEmpty(user.getWebsitId())) {
            AdminWebsit adminWebsit = adminWebsitService.getById(user.getWebsitId());
            userWxBean.setAdminWebsit(adminWebsit);
            if (!StringUtil.isEmpty(lat)){
                userWxBean.setDistance(this.haversine(Convert.toDouble(lat),Convert.toDouble(lng),Convert.toDouble(adminWebsit.getLat()),Convert.toDouble(adminWebsit.getLng())));
            }
        }
        return userWxBean;
    }

    public  double haversine(double lat1, double lon1, double lat2, double lon2) {
        // 将坐标转换为弧度

        int EARTH_RADIUS = 6371;

        double lat1Rad = Math.toRadians(lat1);
        double lon1Rad = Math.toRadians(lon1);
        double lat2Rad = Math.toRadians(lat2);
        double lon2Rad = Math.toRadians(lon2);
        // 计算经纬度差值
        double deltaLat = lat2Rad - lat1Rad;
        double deltaLon = lon2Rad - lon1Rad;
        // Haversine 公式
        double a = Math.sin(deltaLat / 2) * Math.sin(deltaLat / 2) +
                Math.cos(lat1Rad) * Math.cos(lat2Rad) *
                        Math.sin(deltaLon / 2) * Math.sin(deltaLon / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        DecimalFormat df = new DecimalFormat("#.00");
        // 返回距离（单位：公里）
        return Convert.toDouble(df.format((EARTH_RADIUS * c)));
    }


    /**
     * 我的收益
     */
    public MyProfitBean myProfit(String userId) {
        User user = userService.getById(userId);
        if (user == null) {
            return null;
        }
        MyProfitBean myProfitBean = myProfitMapper.queryProfitSum(user.getWorkUserId(),user.getUserId());
        if (myProfitBean != null) {
            myProfitBean.setTotalAmount(myProfitBean.getPaidAmount().add(myProfitBean.getWaitingAmount()));
        } else {
            myProfitBean = new MyProfitBean();
            BigDecimal zero = new BigDecimal("0.00");
            myProfitBean.setPaidAmount(zero);
            myProfitBean.setWaitingAmount(zero);
            myProfitBean.setTotalAmount(zero);
        }
        return myProfitBean;
    }


    public IPage<OrderShare> myProfitList(String userId, String status,Integer pageNum, Integer pageSize) {
        User user = userService.getById(userId);
        if (user == null) {
            return null;
        }
        return orderShareService.lambdaQuery()
                .eq(OrderShare::getOpenId,user.getOpenId())
                .ne(OrderShare::getWorkUserId,"")
                .eq(StringUtils.isNotEmpty(status),OrderShare::getStatus,status)
                .eq(OrderShare::getCompanyWechatId,user.getCompanyId())
                .orderByDesc(OrderShare::getCreateTime)
                .page(new Page<>(pageNum, pageSize));
    }

    /**
     * 总收益
     */
    public BigDecimal totalProfit(String userId) {
        User user = userService.getById(userId);
        if (user == null || StringUtils.isEmpty(user.getWorkUserId())) {
            return null;
        }
        List<OrderShare> list = orderShareService.lambdaQuery()
                .eq(OrderShare::getOpenId,user.getOpenId())
                .eq(OrderShare::getCompanyWechatId,user.getCompanyId())
                .and(v -> v.like(OrderShare::getWorkUserId,user.getWorkUserId()).or().like(OrderShare::getWorkUserId,user.getUserId()))
                .eq(OrderShare::getStatus, OrderShareStatusEnum.OVER.toString())
                .list();
        if (list.size() == 0) {
            return new BigDecimal(0);
        }
        return list.stream().map(OrderShare::getAmount).reduce(BigDecimal::add).orElse(new BigDecimal(0));
    }


    /**
     * 我的客户
     */
    public IPage<User> customerList(String userId, Integer pageNum, Integer pageSize) {
        IPage<User> page = userService.lambdaQuery().eq(User::getServiceId, userId).page(new Page<>(pageNum, pageSize));
        return page;
    }

    /**
     * 我的经理
     */
    public User parentUser(String userId) {
        User user = userService.getById(userId);
        if (StringUtils.isNotEmpty(user.getServiceId())) {
            User parentUser = userService.getById(user.getServiceId());
            return parentUser;
        }
        return null;
    }

    /**
     * 新增用户访问记录
     */
    public void addVisit(HttpServletRequest request, String userId) {
        try {
            CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
            UserVisit userVisit = new UserVisit();
            userVisit.setUserId(userId);
            userVisit.setVisitTime(new Date());
            userVisit.setCompanyId(currentCompanyWechat.getCurrentCompanyWechatId());
            userVisit.setCompanyName(currentCompanyWechat.getCurrentCompanyName());
            userVisit.setWebsitId("");
            userVisit.insert();
        } catch (Exception e) {
            log.error("新增用户访问记录失败", e);
        }
    }


    public AdminWebsit userWebsit(String lng, String lat,HttpServletRequest request) {
        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        User user = userService.getById(currentCompanyWechat.getUserId());
        if (!StringUtil.isEmpty(user.getServiceId())) {
            AdminWebsit adminWebsit = adminWebsitService.getById(user.getWebsitId());
            return adminWebsit;
        }

        AdminWebsit adminWebsit = userInfoMapper.userWebsit(user.getUserId(), lng, lat);

        if (adminWebsit == null)
            return null;

        user.setWebsitId(adminWebsit.getWebsitId());
        user.setWebsitName(adminWebsit.getName());
        user.setCompanyName(adminWebsit.getCompanyName());
        user.setCompanyId(adminWebsit.getCompanyId());
        user.updateById();
        return adminWebsit;
    }

    public List<AdminWebsit> websitList() {
        List<AdminWebsit> adminWebsits = adminWebsitService.lambdaQuery()
                .eq(AdminWebsit::getStatus, "ON").list();
        return adminWebsits;
    }

    public String websitUrl(String websitId,String key, String vrifyCode, HttpServletRequest request) {

        if (org.apache.commons.lang.StringUtils.isNotEmpty(key) && org.apache.commons.lang.StringUtils.isNotEmpty(vrifyCode)) {
            boolean b = this.checkVrifyCode(key, vrifyCode);
            if (!b) {
                throw new RemoteServiceException("验证码不通过");
            }
        }
        AdminWebsit adminWebsit = adminWebsitService.getById(websitId);

        return adminWebsit.getUrl();
    }

    /**
     * 校验验证码
     *
     * @param key
     * @param vrifyCode
     * @return
     */
    private boolean checkVrifyCode(String key, String vrifyCode) throws RemoteServiceException {
        Object serverCode = redisUtil.get(Constant.RedisPrefix.VERIFICATION +":"+key);
        if (serverCode == null) {
            throw new RemoteServiceException("验证码超时");
        }
        //允许拖拽式验证码误差+-10
        int code = (int) serverCode;
        int code2 = Integer.parseInt(vrifyCode);
        if (code >= (code2 - 15) && code <= (code2 + 15)) {
            redisUtil.del(Constant.RedisPrefix.VERIFICATION +":"+key);
            return true;
        } else {
            throw new RemoteServiceException("验证码校验不通过");
        }
    }

    public void getWebsitId(String websitId,HttpServletRequest request) {

        CurrentCompanyWechat currentCompanyWechat = commonLogic.getCurrentCompanyWechat(request);
        User user = userService.getById(currentCompanyWechat.getUserId());
        if (!StringUtil.isEmpty(user.getServiceId())) {
            AdminWebsit adminWebsit = adminWebsitService.getById(user.getWebsitId());
            return;
        }

        AdminWebsit adminWebsit = adminWebsitService.getById(websitId);

        if (adminWebsit == null)
            return;

        user.setWebsitId(adminWebsit.getWebsitId());
        user.setWebsitName(adminWebsit.getName());
        user.setCompanyName(adminWebsit.getCompanyName());
        user.setCompanyId(adminWebsit.getCompanyId());
        user.updateById();
        return ;

    }

    public RegionBean address(HttpServletRequest request, String lat, String lng) {
        RegionBean regionBean = lbsAmapLogic.getRegionByLngLat(lng + "," + lat);
        return regionBean;
    }
}
