package com.gree.mall.miniapp.utils;


import com.gree.mall.miniapp.constant.Constant;
import org.springframework.util.StringUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.List;

public class CommonUtils {


    public static String getAPPID(HttpServletRequest request){
        String header = request.getHeader(Constant.APPID);
        if(StringUtils.isEmpty(header)){
            header = Constant.DEF_APPID;
        }
      //  header = Constant.DEF_APPID;
        return header;
    }

    public static List<String> initList(List<String> list,int size){
        int listSize = list.size();
        if(listSize < size){
            for(int i = listSize;i < size;i++){
                list.add("");
            }
        }
        return list;
    }

    public static String getStatusNameOfStudentExam(int status){
        if(status == 1){
            return "未完成";
        }else if(status == 2){
            return "已完成";
        }else if(status == 0){
            return "作废";
        }
        return "";
    }

    //(1.未考试 2.已及格,3.未及格)
    public static String getExamStatusNameOfStudentExam(int examStatus){
        if(examStatus == 1){
            return "未考试";
        }else if(examStatus == 2){
            return "已及格";
        }else{
            return "未及格";
        }
    }

    public static String getSourceName(int source){
        if(source == 1){
            return "后台新增";
        }else{
            return "线上报名";
        }
    }

    public static Boolean checkNotEmpty(Object... str){
        for(Object s : str){
            if(s instanceof String) {
                String value = (String)s;
                if (StringUtils.isEmpty(value)) {
                    return false;
                }
            }else if(s == null) {
                return false;
            }
        }
        return true;
    }

    public static Boolean checkDate(String dateStr,String dateFormat){
        if(StringUtils.isEmpty(dateStr)){
            return true;
        }
        SimpleDateFormat sdf = new SimpleDateFormat(dateFormat);
        try{
            sdf.parse(dateStr);
        }catch(Exception e){
            return false;
        }
        return true;
    }

    public static String getFormatUrl(String url) {
        if (url.startsWith("http:")) {
            url = StringUtils.replace(url, "http", "https");
        }

        return url;
    }

    /**
     * 从token获取userId
     */
    public static String getUserId(HttpServletRequest request){
        try {
            String token = getToken(request);
            if (StringUtils.isEmpty(token)) {
                token = request.getParameter("token");
            }
            if(StringUtils.isEmpty(token)){
                return null;
            }
            return JwtUtils.validateJWT(token).getClaims().getId();
        }catch(Exception e){
            return "";
        }
    }

    public static String getNickName(HttpServletRequest request){
        try{
            String token = getToken(request);
            if(StringUtils.isEmpty(token))
                return null;
            return JwtUtils.validateJWT(token).getClaims().getSubject();
        }catch(Exception e){
            return "";
        }
    }

    /**
     * 获取token
     */
    public static String getToken(HttpServletRequest request){
        String token = request.getHeader("x-token");
        return token;
    }

    /**
     * 获取性别code
     * @param gender
     * @return
     */
    public static String getGenderCode(String gender){
        if(org.apache.commons.lang3.StringUtils.equals(gender,"男")){
            return "1";
        }else if(org.apache.commons.lang3.StringUtils.equals(gender,"女")){
            return "2";
        }
        //未知
        return "0";
    }

    public static void downloadFile(String fileName, HttpServletResponse response) throws IOException {
        InputStream inputStream  = CommonUtils.class.getResourceAsStream("/mapper/" + fileName);
        response.setContentType("content-type:octet-stream");
        response.setHeader("Content-disposition", "attachment; filename=" + URLEncoder.encode(fileName, "UTF-8"));
        OutputStream outputStream = response.getOutputStream();
        byte[] buffer = new byte[1024];
        int len;
        while ((len = inputStream.read(buffer)) != -1) {
            outputStream.write(buffer, 0, len);
        }
        inputStream.close();
        outputStream.close();
    }


    /**
     * 检查下标对应的值不能为空
     * @param row
     * @param index
     * @return
     */
    public static Integer checkRowEmpty(List<String> row, int[] index){
        for(int i : index){
            if(StringUtils.isEmpty(row.get(i))){
                return i;
            }
        }
        return null;
    }




}
