package com.gree.mall.miniapp.utils.oss;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.common.comm.ResponseMessage;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.model.*;
import com.gree.mall.miniapp.constant.Constant;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;

import java.io.*;
import java.net.URL;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;


@Slf4j
public class OSSUtil {

    // 用户相关文件夹
    public static final String ALIOSS_BUCKETNAME= "zfire-train";
    public static final String OSS_TYPE_PIC = "train/pic/";
    public static final String AliOSSEndPoint = "http://oss-cn-guangzhou.aliyuncs.com";
    public static final String accessUrl = "https://zfire-train.oss-cn-guangzhou.aliyuncs.com/";

    public static final String AliOSSEndPointNohttp = "oss-cn-guangzhou.aliyuncs.com";
//    public static final String OSSAccessKeyID = "LTAI4G3XTLh8G8DiTpXHpWgt";
//    public static final String OSSAccessKeySecret = "LYe8uf7Lg9WxJ1xejk45qnPFqxXLzN";

    private static OSS ossClientPri = new OSSClientBuilder().build(AliOSSEndPoint,
            Constant.Ali.ACCESS_KEY_ID, Constant.Ali.ACCESS_KEY_SECERT);

    public static boolean uploadFile(String ossObjectKey, byte[] file) throws IOException {

        PutObjectResult putObjectResult = null;
        putObjectResult = ossClientPri.putObject(new PutObjectRequest(ALIOSS_BUCKETNAME, ossObjectKey, new ByteArrayInputStream(file)));

        log.info("putObjectResult" + putObjectResult);
        if (putObjectResult != null) {
            ResponseMessage responseMessage = putObjectResult.getResponse();
            if (responseMessage != null) {
                log.error(responseMessage.getErrorResponseAsString());
            }
            return true;
        }

        return false;
    }

    public static boolean uploadFile(String ossObjectKey,File file) throws IOException {
        if (file == null) {
            return false;
        }

        PutObjectResult putObjectResult = ossClientPri.putObject(new PutObjectRequest(ALIOSS_BUCKETNAME, ossObjectKey, new FileInputStream(file)));
        log.info("putObjectResult" + putObjectResult);
        if (putObjectResult != null) {
            ResponseMessage responseMessage = putObjectResult.getResponse();
            if (responseMessage != null) {
                log.error(responseMessage.getErrorResponseAsString());
            }
            return true;
        }
        return false;
    }

    public static Map<String, String> getConfig() throws UnsupportedEncodingException {
        OSS client = new OSSClientBuilder().build(AliOSSEndPoint, Constant.Ali.ACCESS_KEY_ID, Constant.Ali.ACCESS_KEY_SECERT);
        String host = accessUrl; // host的格式为 bucketname.endpoint

//        OSSClient client = new OSSClient(endpoint, accessId, accessKey);
        long expireTime = 300;
        long expireEndTime = System.currentTimeMillis() + expireTime * 1000;
        java.sql.Date expiration = new java.sql.Date(expireEndTime);
        PolicyConditions policyConds = new PolicyConditions();
        policyConds.addConditionItem(PolicyConditions.COND_CONTENT_LENGTH_RANGE, 0, 1048576000);
        policyConds.addConditionItem(MatchMode.StartWith, PolicyConditions.COND_KEY,OSS_TYPE_PIC);

        String postPolicy = client.generatePostPolicy(expiration, policyConds);
        byte[] binaryData = postPolicy.getBytes("utf-8");
        String encodedPolicy = BinaryUtil.toBase64String(binaryData);
        String postSignature = client.calculatePostSignature(postPolicy);

        Map<String, String> respMap = new LinkedHashMap<String, String>();
        respMap.put("OSSAccessKeyId", Constant.Ali.ACCESS_KEY_ID);
        respMap.put("expire", String.valueOf(expireEndTime / 1000));
        respMap.put("policy", encodedPolicy);
        respMap.put("signature", postSignature);
        respMap.put("id",Constant.Ali.ACCESS_KEY_SECERT);
        respMap.put("dir", OSS_TYPE_PIC);
        respMap.put("host", host);
        return respMap;
    }

    public static String getUrlWw(String key){
        OSS clientWw = new OSSClientBuilder().build(AliOSSEndPoint,Constant.Ali.ACCESS_KEY_ID, Constant.Ali.ACCESS_KEY_SECERT);
        // 设置URL过期时间为1小时
        Date expiration = new Date(new Date().getTime() + 3600 * 1000);
        GeneratePresignedUrlRequest generatePresignedUrlRequest ;
        generatePresignedUrlRequest =new GeneratePresignedUrlRequest(ALIOSS_BUCKETNAME, key);
        generatePresignedUrlRequest.setExpiration(expiration);
        URL url = clientWw.generatePresignedUrl(generatePresignedUrlRequest);
        return url.toString();
    }

    public static String getUrlWw(String key, MediaType mediaType) {
        if (org.apache.commons.lang3.StringUtils.isEmpty(key)) {
            return "";
        }
        // 设置URL过期时间为1小时
        Date expiration = new Date(new Date().getTime() + 3600 * 1000);
        GeneratePresignedUrlRequest request = new GeneratePresignedUrlRequest(ALIOSS_BUCKETNAME, key);
        request.setExpiration(expiration);
        if (mediaType != null) {
            request.setContentType(mediaType.toString());
        }

        URL url = new OSSClientBuilder()
                .build("https://" + AliOSSEndPointNohttp, Constant.Ali.ACCESS_KEY_ID, Constant.Ali.ACCESS_KEY_SECERT)
                .generatePresignedUrl(request);

        String resultUrl = url.toString();

        return resultUrl;

    }

}
